package java.io;

import java.security.PrivilegedActionException;
import java.security.PrivilegedExceptionAction;
import java.security.AccessControlContext;
import sun.util.logging.PlatformLogger;
import java.util.concurrent.ConcurrentHashMap;
import sun.security.action.GetIntegerAction;
import sun.security.action.GetBooleanAction;
import java.lang.ref.ReferenceQueue;
import java.util.concurrent.ConcurrentMap;
import sun.misc.SharedSecrets;
import sun.misc.JavaOISAccess;
import sun.misc.VM;
import java.util.Arrays;
import sun.reflect.misc.ReflectUtil;
import java.lang.reflect.Array;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.Objects;
import java.lang.reflect.Proxy;
import sun.misc.ObjectStreamClassValidator;
import sun.misc.ObjectInputFilter;
import java.util.HashMap;

public class ObjectInputStream extends InputStream implements ObjectInput, ObjectStreamConstants
{
    private static final int NULL_HANDLE = -1;
    private static final Object unsharedMarker;
    private static final HashMap<String, Class<?>> primClasses;
    private final ObjectInputStream.BlockDataInputStream bin;
    private final ObjectInputStream.ValidationList vlist;
    private long depth;
    private long totalObjectRefs;
    private boolean closed;
    private final ObjectInputStream.HandleTable handles;
    private int passHandle;
    private boolean defaultDataEnd;
    private byte[] primVals;
    private final boolean enableOverride;
    private boolean enableResolve;
    private SerialCallbackContext curContext;
    private ObjectInputFilter serialFilter;
    private volatile ObjectStreamClassValidator validator;
    
    public ObjectInputStream(InputStream inputStream) throws IOException {
        this.passHandle = -1;
        this.defaultDataEnd = false;
        this.verifySubclass();
        this.bin = new ObjectInputStream.BlockDataInputStream(inputStream);
        this.handles = new ObjectInputStream.HandleTable(10);
        this.vlist = new ObjectInputStream.ValidationList();
        this.serialFilter = ObjectInputFilter.Config.getSerialFilter();
        this.enableOverride = false;
        this.readStreamHeader();
        this.bin.setBlockDataMode(true);
    }
    
    protected ObjectInputStream() throws IOException, SecurityException {
        this.passHandle = -1;
        this.defaultDataEnd = false;
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(ObjectInputStream.SUBCLASS_IMPLEMENTATION_PERMISSION);
        }
        this.bin = null;
        this.handles = null;
        this.vlist = null;
        this.serialFilter = ObjectInputFilter.Config.getSerialFilter();
        this.enableOverride = true;
    }
    
    @Override
    public final Object readObject() throws IOException, ClassNotFoundException {
        return this.readObject(Object.class);
    }
    
    private String readString() throws IOException {
        try {
            return (String)this.readObject(String.class);
        }
        catch (ClassNotFoundException cause) {
            throw new IllegalStateException(cause);
        }
    }
    
    private final Object readObject(Class<?> clazz) throws IOException, ClassNotFoundException {
        if (this.enableOverride) {
            return this.readObjectOverride();
        }
        if (clazz != Object.class && clazz != String.class) {
            throw new AssertionError("internal error");
        }
        int passHandle = this.passHandle;
        try {
            Object object0 = this.readObject0(clazz, false);
            this.handles.markDependency(passHandle, this.passHandle);
            ClassNotFoundException lookupException = this.handles.lookupException(this.passHandle);
            if (lookupException != null) {
                throw lookupException;
            }
            if (this.depth == 0L) {
                this.vlist.doCallbacks();
            }
        }
        finally {
            this.passHandle = passHandle;
            if (this.closed && this.depth == 0L) {
                this.clear();
            }
        }
    }
    
    protected Object readObjectOverride() throws IOException, ClassNotFoundException {
        return null;
    }
    
    public Object readUnshared() throws IOException, ClassNotFoundException {
        int passHandle = this.passHandle;
        try {
            Object object0 = this.readObject0(Object.class, true);
            this.handles.markDependency(passHandle, this.passHandle);
            ClassNotFoundException lookupException = this.handles.lookupException(this.passHandle);
            if (lookupException != null) {
                throw lookupException;
            }
            if (this.depth == 0L) {
                this.vlist.doCallbacks();
            }
        }
        finally {
            this.passHandle = passHandle;
            if (this.closed && this.depth == 0L) {
                this.clear();
            }
        }
    }
    
    public void defaultReadObject() throws IOException, ClassNotFoundException {
        SerialCallbackContext curContext = this.curContext;
        if (curContext == null) {
            throw new NotActiveException("not in call to readObject");
        }
        Object obj = curContext.getObj();
        ObjectStreamClass desc = curContext.getDesc();
        this.bin.setBlockDataMode(false);
        this.defaultReadFields(obj, desc);
        this.bin.setBlockDataMode(true);
        if (!desc.hasWriteObjectData()) {
            this.defaultDataEnd = true;
        }
        ClassNotFoundException lookupException = this.handles.lookupException(this.passHandle);
        if (lookupException != null) {
            throw lookupException;
        }
        return;
    }
    
    public ObjectInputStream.GetField readFields() throws IOException, ClassNotFoundException {
        SerialCallbackContext curContext = this.curContext;
        if (curContext == null) {
            throw new NotActiveException("not in call to readObject");
        }
        curContext.getObj();
        ObjectStreamClass desc = curContext.getDesc();
        this.bin.setBlockDataMode(false);
        ObjectInputStream.GetFieldImpl getFieldImpl = new ObjectInputStream.GetFieldImpl(desc);
        getFieldImpl.readFields();
        this.bin.setBlockDataMode(true);
        if (!desc.hasWriteObjectData()) {
            this.defaultDataEnd = true;
            return getFieldImpl;
        }
        return getFieldImpl;
    }
    
    public void registerValidation(ObjectInputValidation objectInputValidation, int n) throws NotActiveException, InvalidObjectException {
        if (this.depth == 0L) {
            throw new NotActiveException("stream inactive");
        }
        this.vlist.register(objectInputValidation, n);
        return;
    }
    
    protected Class<?> resolveClass(ObjectStreamClass objectStreamClass) throws IOException, ClassNotFoundException {
        String name = objectStreamClass.getName();
        try {
            return Class.forName(name, false, latestUserDefinedLoader());
        }
        catch (ClassNotFoundException ex) {
            Class<Object> clazz = (Class)ObjectInputStream.primClasses.get(name);
            if (clazz != null) {
                return clazz;
            }
            throw ex;
        }
    }
    
    protected Class<?> resolveProxyClass(String[] array) throws IOException, ClassNotFoundException {
        ClassLoader latestUserDefinedLoader = latestUserDefinedLoader();
        ClassLoader classLoader = null;
        int n = 0;
        Class[] interfaces = new Class[array.length];
        Class<Object> forName;
        for (int i = 0; i < array.length; interfaces[i] = forName, ++i) {
            forName = Class.forName(array[i], false, latestUserDefinedLoader);
            if ((forName.getModifiers() & 0x1) == 0x0) {
                if (n != 0) {
                    if (classLoader != forName.getClassLoader()) {
                        throw new IllegalAccessError("conflicting non-public interface class loaders");
                    }
                }
                else {
                    classLoader = forName.getClassLoader();
                    n = 1;
                }
            }
        }
        try {
            return Proxy.getProxyClass((n != 0) ? classLoader : latestUserDefinedLoader, interfaces);
        }
        catch (IllegalArgumentException ex) {
            throw new ClassNotFoundException(null, ex);
        }
    }
    
    protected Object resolveObject(Object o) throws IOException {
        return o;
    }
    
    protected boolean enableResolveObject(boolean enableResolve) throws SecurityException {
        if (enableResolve == this.enableResolve) {
            return enableResolve;
        }
        if (enableResolve) {
            SecurityManager securityManager = System.getSecurityManager();
            if (securityManager != null) {
                securityManager.checkPermission(ObjectInputStream.SUBSTITUTION_PERMISSION);
            }
        }
        this.enableResolve = enableResolve;
        return !this.enableResolve;
    }
    
    protected void readStreamHeader() throws IOException, StreamCorruptedException {
        int short1 = this.bin.readShort();
        int short2 = this.bin.readShort();
        if (short1 != -21267 || short2 != 5) {
            throw new StreamCorruptedException(String.format("invalid stream header: %04X%04X", new Object[] { Short.valueOf((short)short1), Short.valueOf((short)short2) }));
        }
        return;
    }
    
    protected ObjectStreamClass readClassDescriptor() throws IOException, ClassNotFoundException {
        ObjectStreamClass objectStreamClass = new ObjectStreamClass();
        objectStreamClass.readNonProxy(this);
        return objectStreamClass;
    }
    
    @Override
    public int read() throws IOException {
        return this.bin.read();
    }
    
    @Override
    public int read(byte[] array, int n, int n2) throws IOException {
        if (array == null) {
            throw new NullPointerException();
        }
        int n3 = n + n2;
        if (n < 0 || n2 < 0 || n3 > array.length || n3 < 0) {
            throw new IndexOutOfBoundsException();
        }
        return this.bin.read(array, n, n2, false);
    }
    
    @Override
    public int available() throws IOException {
        return this.bin.available();
    }
    
    @Override
    public void close() throws IOException {
        this.closed = true;
        if (this.depth == 0L) {
            this.clear();
        }
        this.bin.close();
    }
    
    @Override
    public boolean readBoolean() throws IOException {
        return this.bin.readBoolean();
    }
    
    @Override
    public byte readByte() throws IOException {
        return this.bin.readByte();
    }
    
    @Override
    public int readUnsignedByte() throws IOException {
        return this.bin.readUnsignedByte();
    }
    
    @Override
    public char readChar() throws IOException {
        return this.bin.readChar();
    }
    
    @Override
    public short readShort() throws IOException {
        return this.bin.readShort();
    }
    
    @Override
    public int readUnsignedShort() throws IOException {
        return this.bin.readUnsignedShort();
    }
    
    @Override
    public int readInt() throws IOException {
        return this.bin.readInt();
    }
    
    @Override
    public long readLong() throws IOException {
        return this.bin.readLong();
    }
    
    @Override
    public float readFloat() throws IOException {
        return this.bin.readFloat();
    }
    
    @Override
    public double readDouble() throws IOException {
        return this.bin.readDouble();
    }
    
    @Override
    public void readFully(byte[] array) throws IOException {
        this.bin.readFully(array, 0, array.length, false);
    }
    
    @Override
    public void readFully(byte[] array, int n, int n2) throws IOException {
        int n3 = n + n2;
        if (n < 0 || n2 < 0 || n3 > array.length || n3 < 0) {
            throw new IndexOutOfBoundsException();
        }
        this.bin.readFully(array, n, n2, false);
        return;
    }
    
    @Override
    public int skipBytes(int n) throws IOException {
        return this.bin.skipBytes(n);
    }
    
    @Deprecated
    @Override
    public String readLine() throws IOException {
        return this.bin.readLine();
    }
    
    @Override
    public String readUTF() throws IOException {
        return this.bin.readUTF();
    }
    
    private final ObjectInputFilter getInternalObjectInputFilter() {
        return this.serialFilter;
    }
    
    private final void setInternalObjectInputFilter(ObjectInputFilter serialFilter) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new SerializablePermission("serialFilter"));
        }
        if (this.serialFilter != null && this.serialFilter != ObjectInputFilter.Config.getSerialFilter()) {
            throw new IllegalStateException("filter can not be set more than once");
        }
        if (this.totalObjectRefs > 0L && !ObjectInputStream.Caches.SET_FILTER_AFTER_READ) {
            throw new IllegalStateException("filter can not be set after an object has been read");
        }
        this.serialFilter = serialFilter;
        return;
    }
    
    private void filterCheck(Class<?> clazz, int n) throws InvalidClassException {
        if (this.serialFilter != null) {
            RuntimeException cause = null;
            long n2 = (this.bin == null) ? 0L : this.bin.getBytesRead();
            ObjectInputFilter.Status obj;
            try {
                obj = this.serialFilter.checkInput(new ObjectInputStream.FilterValues(clazz, (long)n, this.totalObjectRefs, this.depth, n2));
            }
            catch (RuntimeException ex) {
                obj = ObjectInputFilter.Status.REJECTED;
                cause = ex;
            }
            if (obj == null || obj == ObjectInputFilter.Status.REJECTED) {
                if (ObjectInputStream.Logging.infoLogger != null) {
                    ObjectInputStream.Logging.infoLogger.info("ObjectInputFilter {0}: {1}, array length: {2}, nRefs: {3}, depth: {4}, bytes: {5}, ex: {6}", new Object[] { obj, clazz, Integer.valueOf(n), Long.valueOf(this.totalObjectRefs), Long.valueOf(this.depth), Long.valueOf(n2), Objects.toString(cause, "n/a") });
                }
                InvalidClassException o = new InvalidClassException("filter status: " + obj);
                o.initCause(cause);
                throw o;
            }
            else if (ObjectInputStream.Logging.traceLogger != null) {
                ObjectInputStream.Logging.traceLogger.finer("ObjectInputFilter {0}: {1}, array length: {2}, nRefs: {3}, depth: {4}, bytes: {5}, ex: {6}", new Object[] { obj, clazz, Integer.valueOf(n), Long.valueOf(this.totalObjectRefs), Long.valueOf(this.depth), Long.valueOf(n2), Objects.toString(cause, "n/a") });
            }
        }
    }
    
    private void checkArray(Class<?> obj, int n) throws InvalidClassException {
        Objects.requireNonNull(obj);
        if (!obj.isArray()) {
            throw new IllegalArgumentException("not an array type");
        }
        if (n < 0) {
            throw new NegativeArraySizeException();
        }
        this.filterCheck(obj, n);
        return;
    }
    
    private void verifySubclass() {
        Class<Object> class1 = this.getClass();
        if (class1 == ObjectInputStream.class) {
            return;
        }
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager == null) {
            return;
        }
        ObjectStreamClass.processQueue(ObjectInputStream.Caches.subclassAuditsQueue, ObjectInputStream.Caches.subclassAudits);
        ObjectStreamClass.WeakClassKey weakClassKey = new ObjectStreamClass.WeakClassKey(class1, ObjectInputStream.Caches.subclassAuditsQueue);
        Boolean value = (Boolean)ObjectInputStream.Caches.subclassAudits.get(weakClassKey);
        if (value == null) {
            value = Boolean.valueOf(auditSubclass(class1));
            ObjectInputStream.Caches.subclassAudits.putIfAbsent(weakClassKey, value);
        }
        if ((boolean)value) {
            return;
        }
        securityManager.checkPermission(ObjectInputStream.SUBCLASS_IMPLEMENTATION_PERMISSION);
        return;
    }
    
    private static boolean auditSubclass(final Class<?> clazz) {
        return (boolean)AccessController.doPrivileged(new PrivilegedAction<Boolean>() {
            @Override
            public Boolean run() {
                Class<Object> clazz = clazz;
                while (clazz != ObjectInputStream.class) {
                    try {
                        clazz.getDeclaredMethod("readUnshared", (Class[])null);
                        return Boolean.FALSE;
                    }
                    catch (NoSuchMethodException ex) {
                        try {
                            clazz.getDeclaredMethod("readFields", (Class[])null);
                            return Boolean.FALSE;
                        }
                        catch (NoSuchMethodException ex2) {
                            clazz = clazz.getSuperclass();
                        }
                    }
                }
                return Boolean.TRUE;
            }
        });
    }
    
    private void clear() {
        this.handles.clear();
        this.vlist.clear();
    }
    
    private Object readObject0(Class<?> clazz, boolean b) throws IOException {
        int blockDataMode = this.bin.getBlockDataMode();
        if (!(blockDataMode == 0)) {
            int currentBlockRemaining = this.bin.currentBlockRemaining();
            if (currentBlockRemaining > 0) {
                throw new OptionalDataException(currentBlockRemaining);
            }
            if (this.defaultDataEnd) {
                throw new OptionalDataException(true);
            }
            this.bin.setBlockDataMode(false);
        }
        int peekByte;
        while ((peekByte = this.bin.peekByte()) == 121) {
            this.bin.readByte();
            this.handleReset();
        }
        ++this.depth;
        ++this.totalObjectRefs;
        try {
            switch (peekByte) {
                case 112: {
                    return this.readNull();
                }
                case 113: {
                    return clazz.cast(this.readHandle(b));
                }
                case 118: {
                    if (clazz == String.class) {
                        throw new ClassCastException("Cannot cast a class to java.lang.String");
                    }
                    return this.readClass(b);
                }
                case 114:
                case 125: {
                    if (clazz == String.class) {
                        throw new ClassCastException("Cannot cast a class to java.lang.String");
                    }
                    return this.readClassDesc(b);
                }
                case 116:
                case 124: {
                    return this.checkResolve(this.readString(b));
                }
                case 117: {
                    if (clazz == String.class) {
                        throw new ClassCastException("Cannot cast an array to java.lang.String");
                    }
                    return this.checkResolve(this.readArray(b));
                }
                case 126: {
                    if (clazz == String.class) {
                        throw new ClassCastException("Cannot cast an enum to java.lang.String");
                    }
                    return this.checkResolve(this.readEnum(b));
                }
                case 115: {
                    if (clazz == String.class) {
                        throw new ClassCastException("Cannot cast an object to java.lang.String");
                    }
                    return this.checkResolve(this.readOrdinaryObject(b));
                }
                case 123: {
                    if (clazz == String.class) {
                        throw new ClassCastException("Cannot cast an exception to java.lang.String");
                    }
                    throw new WriteAbortedException("writing aborted", this.readFatalException());
                }
                case 119:
                case 122: {
                    if (!(blockDataMode == 0)) {
                        this.bin.setBlockDataMode(true);
                        this.bin.peek();
                        throw new OptionalDataException(this.bin.currentBlockRemaining());
                    }
                    throw new StreamCorruptedException("unexpected block data");
                }
                case 120: {
                    if (!(blockDataMode == 0)) {
                        throw new OptionalDataException(true);
                    }
                    throw new StreamCorruptedException("unexpected end of block data");
                }
                default: {
                    throw new StreamCorruptedException(String.format("invalid type code: %02X", new Object[] { Byte.valueOf((byte)peekByte) }));
                }
            }
        }
        finally {
            --this.depth;
            this.bin.setBlockDataMode((boolean)blockDataMode);
        }
    }
    
    private Object checkResolve(Object o) throws IOException {
        if (!this.enableResolve || this.handles.lookupException(this.passHandle) != null) {
            return o;
        }
        Object resolveObject = this.resolveObject(o);
        if (resolveObject != o) {
            if (resolveObject != null) {
                if (resolveObject.getClass().isArray()) {
                    this.filterCheck(resolveObject.getClass(), Array.getLength(resolveObject));
                }
                else {
                    this.filterCheck(resolveObject.getClass(), -1);
                }
            }
            this.handles.setObject(this.passHandle, resolveObject);
            return resolveObject;
        }
        else {
            return resolveObject;
        }
    }
    
    String readTypeString() throws IOException {
        int passHandle = this.passHandle;
        try {
            int peekByte = this.bin.peekByte();
            switch (peekByte) {
                case 112: {
                    return (String)this.readNull();
                }
                case 113: {
                    return (String)this.readHandle(false);
                }
                case 116:
                case 124: {
                    return this.readString(false);
                }
                default: {
                    throw new StreamCorruptedException(String.format("invalid type code: %02X", new Object[] { Byte.valueOf((byte)peekByte) }));
                }
            }
        }
        finally {
            this.passHandle = passHandle;
        }
    }
    
    private Object readNull() throws IOException {
        if (this.bin.readByte() != 112) {
            throw new InternalError();
        }
        this.passHandle = -1;
        return null;
    }
    
    private Object readHandle(boolean b) throws IOException {
        if (this.bin.readByte() != 113) {
            throw new InternalError();
        }
        this.passHandle = this.bin.readInt() - 8257536;
        if (this.passHandle < 0 || this.passHandle >= this.handles.size()) {
            throw new StreamCorruptedException(String.format("invalid handle value: %08X", new Object[] { Integer.valueOf(this.passHandle + 8257536) }));
        }
        if (b) {
            throw new InvalidObjectException("cannot read back reference as unshared");
        }
        Object lookupObject = this.handles.lookupObject(this.passHandle);
        if (lookupObject == ObjectInputStream.unsharedMarker) {
            throw new InvalidObjectException("cannot read back reference to unshared object");
        }
        this.filterCheck(null, -1);
        return lookupObject;
    }
    
    private Class<?> readClass(boolean b) throws IOException {
        if (this.bin.readByte() != 118) {
            throw new InternalError();
        }
        ObjectStreamClass classDesc = this.readClassDesc(false);
        Class<Object> forClass = classDesc.forClass();
        this.passHandle = this.handles.assign(b ? ObjectInputStream.unsharedMarker : forClass);
        ClassNotFoundException resolveException = classDesc.getResolveException();
        if (resolveException != null) {
            this.handles.markException(this.passHandle, resolveException);
        }
        this.handles.finish(this.passHandle);
        return forClass;
    }
    
    private ObjectStreamClass readClassDesc(boolean b) throws IOException {
        int peekByte = this.bin.peekByte();
        ObjectStreamClass objectStreamClass;
        switch (peekByte) {
            case 112: {
                objectStreamClass = (ObjectStreamClass)this.readNull();
                break;
            }
            case 113: {
                objectStreamClass = (ObjectStreamClass)this.readHandle(b);
                objectStreamClass.checkInitialized();
                break;
            }
            case 125: {
                objectStreamClass = this.readProxyDesc(b);
                break;
            }
            case 114: {
                objectStreamClass = this.readNonProxyDesc(b);
                break;
            }
            default: {
                throw new StreamCorruptedException(String.format("invalid type code: %02X", new Object[] { Byte.valueOf((byte)peekByte) }));
            }
        }
        if (objectStreamClass != null) {
            this.validateDescriptor(objectStreamClass);
            return objectStreamClass;
        }
        return objectStreamClass;
    }
    
    private boolean isCustomSubclass() {
        return this.getClass().getClassLoader() != ObjectInputStream.class.getClassLoader();
    }
    
    private ObjectStreamClass readProxyDesc(boolean b) throws IOException {
        if (this.bin.readByte() != 125) {
            throw new InternalError();
        }
        ObjectStreamClass objectStreamClass = new ObjectStreamClass();
        int assign = this.handles.assign(b ? ObjectInputStream.unsharedMarker : objectStreamClass);
        this.passHandle = -1;
        int int1 = this.bin.readInt();
        if (int1 > 65535) {
            throw new InvalidObjectException("interface limit exceeded: " + int1 + ", limit: " + ObjectInputStream.Caches.PROXY_INTERFACE_LIMIT);
        }
        String[] a = new String[int1];
        for (int i = 0; i < int1; ++i) {
            a[i] = this.bin.readUTF();
        }
        if (int1 > ObjectInputStream.Caches.PROXY_INTERFACE_LIMIT) {
            throw new InvalidObjectException("interface limit exceeded: " + int1 + ", limit: " + ObjectInputStream.Caches.PROXY_INTERFACE_LIMIT + "; " + Arrays.toString(a));
        }
        Class<Object> resolveProxyClass = null;
        ClassNotFoundException ex = null;
        this.bin.setBlockDataMode(true);
        try {
            if ((resolveProxyClass = this.resolveProxyClass(a)) == null) {
                ex = new ClassNotFoundException("null class");
            }
            else {
                if (!Proxy.isProxyClass(resolveProxyClass)) {
                    throw new InvalidClassException("Not a proxy");
                }
                ReflectUtil.checkProxyPackageAccess(this.getClass().getClassLoader(), resolveProxyClass.getInterfaces());
                Class[] interfaces = resolveProxyClass.getInterfaces();
                for (int length = interfaces.length, j = 0; j < length; ++j) {
                    this.filterCheck(interfaces[j], -1);
                }
            }
        }
        catch (ClassNotFoundException ex2) {
            ex = ex2;
        }
        catch (OutOfMemoryError cause) {
            InvalidObjectException ex3 = new InvalidObjectException("Proxy interface limit exceeded: " + Arrays.toString(a));
            ex3.initCause(cause);
            throw ex3;
        }
        this.filterCheck(resolveProxyClass, -1);
        this.skipCustomData();
        try {
            ++this.totalObjectRefs;
            ++this.depth;
            objectStreamClass.initProxy(resolveProxyClass, ex, this.readClassDesc(false));
        }
        catch (OutOfMemoryError cause2) {
            InvalidObjectException o = new InvalidObjectException("Proxy interface limit exceeded: " + Arrays.toString(a));
            o.initCause(cause2);
            throw o;
        }
        finally {
            --this.depth;
        }
        this.handles.finish(assign);
        this.passHandle = assign;
        return objectStreamClass;
    }
    
    private ObjectStreamClass readNonProxyDesc(boolean b) throws IOException {
        if (this.bin.readByte() != 114) {
            throw new InternalError();
        }
        ObjectStreamClass objectStreamClass = new ObjectStreamClass();
        int assign = this.handles.assign(b ? ObjectInputStream.unsharedMarker : objectStreamClass);
        this.passHandle = -1;
        ObjectStreamClass classDescriptor;
        try {
            classDescriptor = this.readClassDescriptor();
        }
        catch (ClassNotFoundException cause) {
            throw (IOException)new InvalidClassException("failed to read class descriptor").initCause(cause);
        }
        Class<Object> resolveClass = null;
        ClassNotFoundException ex = null;
        this.bin.setBlockDataMode(true);
        int customSubclass = this.isCustomSubclass();
        try {
            if ((resolveClass = this.resolveClass(classDescriptor)) == null) {
                ex = new ClassNotFoundException("null class");
            }
            else if (!(customSubclass == 0)) {
                ReflectUtil.checkPackageAccess(resolveClass);
            }
        }
        catch (ClassNotFoundException ex2) {
            ex = ex2;
        }
        this.filterCheck(resolveClass, -1);
        this.skipCustomData();
        try {
            ++this.totalObjectRefs;
            ++this.depth;
            objectStreamClass.initNonProxy(classDescriptor, resolveClass, ex, this.readClassDesc(false));
        }
        finally {
            --this.depth;
        }
        this.handles.finish(assign);
        this.passHandle = assign;
        return objectStreamClass;
    }
    
    private String readString(boolean b) throws IOException {
        int byte1 = this.bin.readByte();
        String s;
        switch (byte1) {
            case 116: {
                s = this.bin.readUTF();
                break;
            }
            case 124: {
                s = this.bin.readLongUTF();
                break;
            }
            default: {
                throw new StreamCorruptedException(String.format("invalid type code: %02X", new Object[] { Byte.valueOf((byte)byte1) }));
            }
        }
        this.passHandle = this.handles.assign(b ? ObjectInputStream.unsharedMarker : s);
        this.handles.finish(this.passHandle);
        return s;
    }
    
    private Object readArray(boolean b) throws IOException {
        if (this.bin.readByte() != 117) {
            throw new InternalError();
        }
        ObjectStreamClass classDesc = this.readClassDesc(false);
        int int1 = this.bin.readInt();
        this.filterCheck(classDesc.forClass(), int1);
        Object instance = null;
        Class<Object> componentType = null;
        Class<Object> forClass;
        if ((forClass = classDesc.forClass()) != null) {
            componentType = forClass.getComponentType();
            instance = Array.newInstance(componentType, int1);
        }
        int assign = this.handles.assign(b ? ObjectInputStream.unsharedMarker : instance);
        ClassNotFoundException resolveException = classDesc.getResolveException();
        if (resolveException != null) {
            this.handles.markException(assign, resolveException);
        }
        if (componentType == null) {
            for (int j = 0; j < int1; ++j) {
                this.readObject0(Object.class, false);
            }
        }
        else if (componentType.isPrimitive()) {
            if (componentType == Integer.TYPE) {
                this.bin.readInts((int[])(int[])instance, 0, int1);
            }
            else if (componentType == Byte.TYPE) {
                this.bin.readFully((byte[])(byte[])instance, 0, int1, true);
            }
            else if (componentType == Long.TYPE) {
                this.bin.readLongs((long[])(long[])instance, 0, int1);
            }
            else if (componentType == Float.TYPE) {
                this.bin.readFloats((float[])(float[])instance, 0, int1);
            }
            else if (componentType == Double.TYPE) {
                this.bin.readDoubles((double[])(double[])instance, 0, int1);
            }
            else if (componentType == Short.TYPE) {
                this.bin.readShorts((short[])(short[])instance, 0, int1);
            }
            else if (componentType == Character.TYPE) {
                this.bin.readChars((char[])(char[])instance, 0, int1);
            }
            else if (componentType == Boolean.TYPE) {
                this.bin.readBooleans((boolean[])(boolean[])instance, 0, int1);
            }
            else {
                throw new InternalError();
            }
        }
        else {
            Object[] array = (Object[])(Object[])instance;
            for (int i = 0; i < int1; ++i) {
                array[i] = this.readObject0(Object.class, false);
                this.handles.markDependency(assign, this.passHandle);
            }
        }
        this.handles.finish(assign);
        this.passHandle = assign;
        return instance;
    }
    
    private Enum<?> readEnum(boolean b) throws IOException {
        if (this.bin.readByte() != 126) {
            throw new InternalError();
        }
        ObjectStreamClass classDesc = this.readClassDesc(false);
        if (!classDesc.isEnum()) {
            throw new InvalidClassException("non-enum class: " + classDesc);
        }
        int assign = this.handles.assign(b ? ObjectInputStream.unsharedMarker : null);
        ClassNotFoundException resolveException = classDesc.getResolveException();
        if (resolveException != null) {
            this.handles.markException(assign, resolveException);
        }
        String string = this.readString(false);
        Enum<Object> value = null;
        Class<Enum> forClass = classDesc.forClass();
        if (forClass != null) {
            try {
                value = Enum.valueOf(forClass, string);
            }
            catch (IllegalArgumentException cause) {
                throw (IOException)new InvalidObjectException("enum constant " + string + " does not exist in " + forClass).initCause(cause);
            }
            if (!b) {
                this.handles.setObject(assign, value);
            }
        }
        this.handles.finish(assign);
        this.passHandle = assign;
        return value;
    }
    
    private Object readOrdinaryObject(boolean b) throws IOException {
        if (this.bin.readByte() != 115) {
            throw new InternalError();
        }
        ObjectStreamClass classDesc = this.readClassDesc(false);
        classDesc.checkDeserialize();
        Class<Object> forClass = classDesc.forClass();
        if (forClass == String.class || forClass == Class.class || forClass == ObjectStreamClass.class) {
            throw new InvalidClassException("invalid class descriptor");
        }
        try {
            Object o = classDesc.isInstantiable() ? classDesc.newInstance() : null;
        }
        catch (Exception cause) {
            throw (IOException)new InvalidClassException(classDesc.forClass().getName(), "unable to create instance").initCause(cause);
        }
        Object o2;
        this.passHandle = this.handles.assign(b ? ObjectInputStream.unsharedMarker : o2);
        ClassNotFoundException resolveException = classDesc.getResolveException();
        if (resolveException != null) {
            this.handles.markException(this.passHandle, resolveException);
        }
        if (classDesc.isExternalizable()) {
            this.readExternalData((Externalizable)o2, classDesc);
        }
        else {
            this.readSerialData(o2, classDesc);
        }
        this.handles.finish(this.passHandle);
        if (o2 != null && this.handles.lookupException(this.passHandle) == null && classDesc.hasReadResolveMethod()) {
            o2 = classDesc.invokeReadResolve(o2);
            if (b && o2.getClass().isArray()) {
                o2 = cloneArray(o2);
            }
            if (o2 != o2) {
                if (o2 != null) {
                    if (o2.getClass().isArray()) {
                        this.filterCheck(o2.getClass(), Array.getLength(o2));
                    }
                    else {
                        this.filterCheck(o2.getClass(), -1);
                    }
                }
                this.handles.setObject(this.passHandle, o2);
            }
        }
        return o2;
    }
    
    private void readExternalData(Externalizable externalizable, ObjectStreamClass objectStreamClass) throws IOException {
        SerialCallbackContext curContext = this.curContext;
        if (curContext != null) {
            curContext.check();
        }
        this.curContext = null;
        try {
            int hasBlockExternalData = objectStreamClass.hasBlockExternalData();
            if (!(hasBlockExternalData == 0)) {
                this.bin.setBlockDataMode(true);
            }
            if (externalizable != null) {
                try {
                    externalizable.readExternal(this);
                }
                catch (ClassNotFoundException ex) {
                    this.handles.markException(this.passHandle, ex);
                }
            }
            if (!(hasBlockExternalData == 0)) {
                this.skipCustomData();
            }
        }
        finally {
            if (curContext != null) {
                curContext.check();
            }
            this.curContext = curContext;
        }
    }
    
    private void readSerialData(Object o, ObjectStreamClass objectStreamClass) throws IOException {
        ObjectStreamClass.ClassDataSlot[] classDataLayout = objectStreamClass.getClassDataLayout();
        for (int i = 0; i < classDataLayout.length; ++i) {
            ObjectStreamClass desc = classDataLayout[i].desc;
            if (classDataLayout[i].hasData) {
                if (o == null || this.handles.lookupException(this.passHandle) != null) {
                    this.defaultReadFields(null, desc);
                }
                else if (desc.hasReadObjectMethod()) {
                    ThreadDeath threadDeath = null;
                    int n = 0;
                    SerialCallbackContext curContext = this.curContext;
                    if (curContext != null) {
                        curContext.check();
                    }
                    try {
                        this.curContext = new SerialCallbackContext(o, desc);
                        this.bin.setBlockDataMode(true);
                        desc.invokeReadObject(o, this);
                        while (true) {
                            try {
                                this.curContext.setUsed();
                                if (curContext != null) {
                                    curContext.check();
                                }
                                this.curContext = curContext;
                                n = 1;
                            }
                            catch (ThreadDeath threadDeath2) {
                                threadDeath = threadDeath2;
                            }
                            if (n != 0) {
                                break;
                            }
                            continue;
                        }
                        if (threadDeath != null) {
                            throw threadDeath;
                        }
                    }
                    catch (ClassNotFoundException ex) {
                        this.handles.markException(this.passHandle, ex);
                        while (true) {
                            try {
                                this.curContext.setUsed();
                                if (curContext != null) {
                                    curContext.check();
                                }
                                this.curContext = curContext;
                                n = 1;
                            }
                            catch (ThreadDeath threadDeath3) {
                                threadDeath = threadDeath3;
                            }
                            if (n != 0) {
                                break;
                            }
                            continue;
                        }
                        if (threadDeath != null) {
                            throw threadDeath;
                        }
                    }
                    finally {
                        while (true) {
                            try {
                                this.curContext.setUsed();
                                if (curContext != null) {
                                    curContext.check();
                                }
                                this.curContext = curContext;
                                n = 1;
                            }
                            catch (ThreadDeath threadDeath4) {
                                threadDeath = threadDeath4;
                            }
                            if (n != 0) {
                                break;
                            }
                            continue;
                        }
                        if (threadDeath != null) {
                            throw threadDeath;
                        }
                    }
                    this.defaultDataEnd = false;
                }
                else {
                    this.defaultReadFields(o, desc);
                }
                if (desc.hasWriteObjectData()) {
                    this.skipCustomData();
                }
                else {
                    this.bin.setBlockDataMode(false);
                }
            }
            else if (o != null && desc.hasReadObjectNoDataMethod() && this.handles.lookupException(this.passHandle) == null) {
                desc.invokeReadObjectNoData(o);
            }
        }
    }
    
    private void skipCustomData() throws IOException {
        int passHandle = this.passHandle;
    Label_0080:
        while (true) {
            if (this.bin.getBlockDataMode()) {
                this.bin.skipBlockData();
                this.bin.setBlockDataMode(false);
            }
            switch (this.bin.peekByte()) {
                case 119:
                case 122: {
                    this.bin.setBlockDataMode(true);
                    continue;
                }
                case 120: {
                    break Label_0080;
                }
                default: {
                    this.readObject0(Object.class, false);
                    continue;
                }
            }
        }
        this.bin.readByte();
        this.passHandle = passHandle;
    }
    
    private void defaultReadFields(Object o, ObjectStreamClass objectStreamClass) throws IOException {
        Class<Object> forClass = objectStreamClass.forClass();
        if (forClass != null && o != null && !forClass.isInstance(o)) {
            throw new ClassCastException();
        }
        int primDataSize = objectStreamClass.getPrimDataSize();
        if (this.primVals == null || this.primVals.length < primDataSize) {
            this.primVals = new byte[primDataSize];
        }
        this.bin.readFully(this.primVals, 0, primDataSize, false);
        if (o != null) {
            objectStreamClass.setPrimFieldValues(o, this.primVals);
        }
        int passHandle = this.passHandle;
        ObjectStreamField[] fields = objectStreamClass.getFields(false);
        Object[] array = new Object[objectStreamClass.getNumObjFields()];
        int n = fields.length - array.length;
        for (int i = 0; i < array.length; ++i) {
            ObjectStreamField objectStreamField = fields[n + i];
            array[i] = this.readObject0(Object.class, objectStreamField.isUnshared());
            if (objectStreamField.getField() != null) {
                this.handles.markDependency(passHandle, this.passHandle);
            }
        }
        if (o != null) {
            objectStreamClass.setObjFieldValues(o, array);
        }
        this.passHandle = passHandle;
        return;
    }
    
    private IOException readFatalException() throws IOException {
        if (this.bin.readByte() != 123) {
            throw new InternalError();
        }
        this.clear();
        return (IOException)this.readObject0(Object.class, false);
    }
    
    private void handleReset() throws StreamCorruptedException {
        if (this.depth > 0L) {
            throw new StreamCorruptedException("unexpected reset; recursion depth: " + this.depth);
        }
        this.clear();
        return;
    }
    
    private static native void bytesToFloats(byte[] p0, int p1, float[] p2, int p3, int p4);
    
    private static native void bytesToDoubles(byte[] p0, int p1, double[] p2, int p3, int p4);
    
    private static ClassLoader latestUserDefinedLoader() {
        return VM.latestUserDefinedLoader();
    }
    
    private static Object cloneArray(Object o) {
        if ((o instanceof Object[])) {
            return ((Object[])(Object[])o).clone();
        }
        if ((o instanceof boolean[])) {
            return ((boolean[])(boolean[])o).clone();
        }
        if ((o instanceof byte[])) {
            return ((byte[])(byte[])o).clone();
        }
        if ((o instanceof char[])) {
            return ((char[])(char[])o).clone();
        }
        if ((o instanceof double[])) {
            return ((double[])(double[])o).clone();
        }
        if ((o instanceof float[])) {
            return ((float[])(float[])o).clone();
        }
        if ((o instanceof int[])) {
            return ((int[])(int[])o).clone();
        }
        if ((o instanceof long[])) {
            return ((long[])(long[])o).clone();
        }
        if ((o instanceof short[])) {
            return ((short[])(short[])o).clone();
        }
        throw new AssertionError();
    }
    
    private void validateDescriptor(ObjectStreamClass objectStreamClass) {
        ObjectStreamClassValidator validator = this.validator;
        if (validator != null) {
            validator.validateDescriptor(objectStreamClass);
        }
    }
    
    private static void setValidator(ObjectInputStream objectInputStream, ObjectStreamClassValidator validator) {
        objectInputStream.validator = validator;
    }
    
    static void access$000(ObjectInputStream objectInputStream, ObjectInputFilter internalObjectInputFilter) {
        objectInputStream.setInternalObjectInputFilter(internalObjectInputFilter);
    }
    
    static ObjectInputFilter access$100(ObjectInputStream objectInputStream) {
        return objectInputStream.getInternalObjectInputFilter();
    }
    
    static void access$200(ObjectInputStream objectInputStream, Class clazz, int n) throws InvalidClassException {
        objectInputStream.checkArray(clazz, n);
    }
    
    static Object access$800(ObjectInputStream objectInputStream, Class clazz, boolean b) throws IOException {
        return objectInputStream.readObject0(clazz, b);
    }
    
    static void access$1100(ObjectInputStream objectInputStream) throws StreamCorruptedException {
        objectInputStream.handleReset();
    }
    
    static void access$1200(byte[] array, int n, float[] array2, int n2, int n3) {
        bytesToFloats(array, n, array2, n2, n3);
    }
    
    static void access$1300(byte[] array, int n, double[] array2, int n2, int n3) {
        bytesToDoubles(array, n, array2, n2, n3);
    }
    
    static {
        unsharedMarker = new Object();
        (primClasses = new HashMap(8, 1.0f)).put("boolean", Boolean.TYPE);
        ObjectInputStream.primClasses.put("byte", Byte.TYPE);
        ObjectInputStream.primClasses.put("char", Character.TYPE);
        ObjectInputStream.primClasses.put("short", Short.TYPE);
        ObjectInputStream.primClasses.put("int", Integer.TYPE);
        ObjectInputStream.primClasses.put("long", Long.TYPE);
        ObjectInputStream.primClasses.put("float", Float.TYPE);
        ObjectInputStream.primClasses.put("double", Double.TYPE);
        ObjectInputStream.primClasses.put("void", Void.TYPE);
        SharedSecrets.setJavaOISAccess(new JavaOISAccess() {
            public void setObjectInputFilter(ObjectInputStream objectInputStream, ObjectInputFilter objectInputFilter) {
                ObjectInputStream.access$000(objectInputStream, objectInputFilter);
            }
            
            public ObjectInputFilter getObjectInputFilter(ObjectInputStream objectInputStream) {
                return ObjectInputStream.access$100(objectInputStream);
            }
            
            public void checkArray(ObjectInputStream objectInputStream, Class<?> clazz, int n) throws InvalidClassException {
                ObjectInputStream.access$200(objectInputStream, clazz, n);
            }
        });
        SharedSecrets.setJavaObjectInputStreamAccess(ObjectInputStream::setValidator);
        SharedSecrets.setJavaObjectInputStreamReadString(ObjectInputStream::readString);
    }
    
    private static class Caches
    {
        static final ConcurrentMap<ObjectStreamClass.WeakClassKey, Boolean> subclassAudits;
        static final ReferenceQueue<Class<?>> subclassAuditsQueue;
        static final boolean SET_FILTER_AFTER_READ;
        static final int PROXY_INTERFACE_LIMIT;
        
        private static boolean privilegedGetProperty(String s) {
            if (System.getSecurityManager() == null) {
                return Boolean.getBoolean(s);
            }
            return (boolean)AccessController.doPrivileged(new GetBooleanAction(s));
        }
        
        private static Integer privilegedGetProperty(String s, int n) {
            if (System.getSecurityManager() == null) {
                return Integer.getInteger(s, n);
            }
            return (Integer)AccessController.doPrivileged(new GetIntegerAction(s, n));
        }
        
        static {
            subclassAudits = new ConcurrentHashMap();
            subclassAuditsQueue = new ReferenceQueue();
            SET_FILTER_AFTER_READ = privilegedGetProperty("jdk.serialSetFilterAfterRead");
            PROXY_INTERFACE_LIMIT = Math.max(0, Math.min(65535, (int)privilegedGetProperty("jdk.serialProxyInterfaceLimit", 65535)));
        }
    }
    
    private static class Logging
    {
        private static final PlatformLogger traceLogger;
        private static final PlatformLogger infoLogger;
        
        static {
            PlatformLogger logger = PlatformLogger.getLogger("java.io.serialization");
            infoLogger = ((logger != null && logger.isLoggable(PlatformLogger.Level.INFO)) ? logger : null);
            traceLogger = ((logger != null && logger.isLoggable(PlatformLogger.Level.FINER)) ? logger : null);
        }
    }
    
    public abstract static class GetField
    {
        public abstract ObjectStreamClass getObjectStreamClass();
        
        public abstract boolean defaulted(String p0) throws IOException;
        
        public abstract boolean get(String p0, boolean p1) throws IOException;
        
        public abstract byte get(String p0, byte p1) throws IOException;
        
        public abstract char get(String p0, char p1) throws IOException;
        
        public abstract short get(String p0, short p1) throws IOException;
        
        public abstract int get(String p0, int p1) throws IOException;
        
        public abstract long get(String p0, long p1) throws IOException;
        
        public abstract float get(String p0, float p1) throws IOException;
        
        public abstract double get(String p0, double p1) throws IOException;
        
        public abstract Object get(String p0, Object p1) throws IOException;
    }
    
    private class GetFieldImpl extends ObjectInputStream.GetField
    {
        private final ObjectStreamClass desc;
        private final byte[] primVals;
        private final Object[] objVals;
        private final int[] objHandles;
        
        GetFieldImpl(ObjectStreamClass desc) {
            this.desc = desc;
            this.primVals = new byte[desc.getPrimDataSize()];
            this.objVals = new Object[desc.getNumObjFields()];
            this.objHandles = new int[this.objVals.length];
        }
        
        @Override
        public ObjectStreamClass getObjectStreamClass() {
            return this.desc;
        }
        
        @Override
        public boolean defaulted(String s) throws IOException {
            return this.getFieldOffset(s, null) < 0;
        }
        
        @Override
        public boolean get(String s, boolean b) throws IOException {
            int fieldOffset = this.getFieldOffset(s, Boolean.TYPE);
            return (fieldOffset >= 0) ? Bits.getBoolean(this.primVals, fieldOffset) : b;
        }
        
        @Override
        public byte get(String s, byte b) throws IOException {
            int fieldOffset = this.getFieldOffset(s, Byte.TYPE);
            return (fieldOffset >= 0) ? this.primVals[fieldOffset] : b;
        }
        
        @Override
        public char get(String s, char c) throws IOException {
            int fieldOffset = this.getFieldOffset(s, Character.TYPE);
            return (fieldOffset >= 0) ? Bits.getChar(this.primVals, fieldOffset) : c;
        }
        
        @Override
        public short get(String s, short n) throws IOException {
            int fieldOffset = this.getFieldOffset(s, Short.TYPE);
            return (fieldOffset >= 0) ? Bits.getShort(this.primVals, fieldOffset) : n;
        }
        
        @Override
        public int get(String s, int n) throws IOException {
            int fieldOffset = this.getFieldOffset(s, Integer.TYPE);
            return (fieldOffset >= 0) ? Bits.getInt(this.primVals, fieldOffset) : n;
        }
        
        @Override
        public float get(String s, float n) throws IOException {
            int fieldOffset = this.getFieldOffset(s, Float.TYPE);
            return (fieldOffset >= 0) ? Bits.getFloat(this.primVals, fieldOffset) : n;
        }
        
        @Override
        public long get(String s, long n) throws IOException {
            int fieldOffset = this.getFieldOffset(s, Long.TYPE);
            return (fieldOffset >= 0) ? Bits.getLong(this.primVals, fieldOffset) : n;
        }
        
        @Override
        public double get(String s, double n) throws IOException {
            int fieldOffset = this.getFieldOffset(s, Double.TYPE);
            return (fieldOffset >= 0) ? Bits.getDouble(this.primVals, fieldOffset) : n;
        }
        
        @Override
        public Object get(String s, Object o) throws IOException {
            int fieldOffset = this.getFieldOffset(s, Object.class);
            if (fieldOffset >= 0) {
                int n = this.objHandles[fieldOffset];
                ObjectInputStream.this.handles.markDependency(ObjectInputStream.this.passHandle, n);
                return (ObjectInputStream.this.handles.lookupException(n) == null) ? this.objVals[fieldOffset] : null;
            }
            return o;
        }
        
        void readFields() throws IOException {
            ObjectInputStream.this.bin.readFully(this.primVals, 0, this.primVals.length, false);
            int access$500 = ObjectInputStream.this.passHandle;
            ObjectStreamField[] fields = this.desc.getFields(false);
            int n = fields.length - this.objVals.length;
            for (int i = 0; i < this.objVals.length; ++i) {
                this.objVals[i] = ObjectInputStream.access$800(ObjectInputStream.this, Object.class, fields[n + i].isUnshared());
                this.objHandles[i] = ObjectInputStream.this.passHandle;
            }
            ObjectInputStream.this.passHandle = access$500;
        }
        
        private int getFieldOffset(String str, Class<?> obj) {
            ObjectStreamField field = this.desc.getField(str, obj);
            if (field != null) {
                return field.getOffset();
            }
            if (this.desc.getLocalDesc().getField(str, obj) != null) {
                return -1;
            }
            throw new IllegalArgumentException("no such field " + str + " with type " + obj);
        }
    }
    
    private static class ValidationList
    {
        private ObjectInputStream.ValidationList.Callback list;
        
        ValidationList() {
        }
        
        void register(ObjectInputValidation objectInputValidation, int n) throws InvalidObjectException {
            if (objectInputValidation == null) {
                throw new InvalidObjectException("null callback");
            }
            ObjectInputStream.ValidationList.Callback callback = null;
            ObjectInputStream.ValidationList.Callback callback2;
            for (callback2 = this.list; callback2 != null && n < callback2.priority; callback2 = callback2.next) {
                callback = callback2;
            }
            AccessControlContext context = AccessController.getContext();
            if (callback != null) {
                callback.next = new ObjectInputStream.ValidationList.Callback(objectInputValidation, n, callback2, context);
            }
            else {
                this.list = new ObjectInputStream.ValidationList.Callback(objectInputValidation, n, this.list, context);
            }
            return;
        }
        
        void doCallbacks() throws InvalidObjectException {
            try {
                while (this.list != null) {
                    AccessController.doPrivileged(new PrivilegedExceptionAction<Void>() {
                        @Override
                        public Void run() throws InvalidObjectException {
                            ObjectInputStream.ValidationList.this.list.obj.validateObject();
                            return null;
                        }
                    }, this.list.acc);
                    this.list = this.list.next;
                }
            }
            catch (PrivilegedActionException ex) {
                this.list = null;
                throw (InvalidObjectException)ex.getException();
            }
        }
        
        public void clear() {
            this.list = null;
        }
        
        private static class Callback
        {
            final ObjectInputValidation obj;
            final int priority;
            ObjectInputStream.ValidationList.Callback next;
            final AccessControlContext acc;
            
            Callback(ObjectInputValidation obj, int priority, ObjectInputStream.ValidationList.Callback next, AccessControlContext acc) {
                this.obj = obj;
                this.priority = priority;
                this.next = next;
                this.acc = acc;
            }
        }
    }
    
    static class FilterValues implements ObjectInputFilter.FilterInfo
    {
        final Class<?> clazz;
        final long arrayLength;
        final long totalObjectRefs;
        final long depth;
        final long streamBytes;
        
        public FilterValues(Class<?> clazz, long arrayLength, long totalObjectRefs, long depth, long streamBytes) {
            this.clazz = clazz;
            this.arrayLength = arrayLength;
            this.totalObjectRefs = totalObjectRefs;
            this.depth = depth;
            this.streamBytes = streamBytes;
        }
        
        public Class<?> serialClass() {
            return this.clazz;
        }
        
        public long arrayLength() {
            return this.arrayLength;
        }
        
        public long references() {
            return this.totalObjectRefs;
        }
        
        public long depth() {
            return this.depth;
        }
        
        public long streamBytes() {
            return this.streamBytes;
        }
    }
    
    private static class PeekInputStream extends InputStream
    {
        private final InputStream in;
        private int peekb;
        private long totalBytesRead;
        
        PeekInputStream(InputStream in) {
            this.peekb = -1;
            this.totalBytesRead = 0L;
            this.in = in;
        }
        
        int peek() throws IOException {
            if (this.peekb >= 0) {
                return this.peekb;
            }
            this.peekb = this.in.read();
            this.totalBytesRead += (this.peekb >= 0);
            return this.peekb;
        }
        
        @Override
        public int read() throws IOException {
            if (this.peekb >= 0) {
                int peekb = this.peekb;
                this.peekb = -1;
                return peekb;
            }
            int read = this.in.read();
            this.totalBytesRead += (read >= 0);
            return read;
        }
        
        @Override
        public int read(byte[] array, int n, int n2) throws IOException {
            if (n2 == 0) {
                return 0;
            }
            if (this.peekb < 0) {
                int read2 = this.in.read(array, n, n2);
                this.totalBytesRead += ((read2 >= 0) ? ((long)read2) : 0L);
                return read2;
            }
            array[n++] = (byte)this.peekb;
            --n2;
            this.peekb = -1;
            int read = this.in.read(array, n, n2);
            this.totalBytesRead += ((read >= 0) ? ((long)read) : 0L);
            return (read >= 0) ? (read + 1) : 1;
        }
        
        void readFully(byte[] array, int n, int n2) throws IOException {
            int i = 0;
            while (i < n2) {
                int read = this.read(array, n + i, n2 - i);
                if (read < 0) {
                    throw new EOFException();
                }
                i += read;
            }
        }
        
        @Override
        public long skip(long n) throws IOException {
            if (n <= 0L) {
                return 0L;
            }
            int n2 = 0;
            if (this.peekb >= 0) {
                this.peekb = -1;
                ++n2;
                --n;
            }
            n = (long)n2 + this.in.skip(n);
            this.totalBytesRead += n;
            return n;
        }
        
        @Override
        public int available() throws IOException {
            return this.in.available() + ((this.peekb >= 0) ? 1 : 0);
        }
        
        @Override
        public void close() throws IOException {
            this.in.close();
        }
        
        public long getBytesRead() {
            return this.totalBytesRead;
        }
    }
    
    private class BlockDataInputStream extends InputStream implements DataInput
    {
        private static final int MAX_BLOCK_SIZE = 1024;
        private static final int MAX_HEADER_SIZE = 5;
        private static final int CHAR_BUF_SIZE = 256;
        private static final int HEADER_BLOCKED = -2;
        private final byte[] buf;
        private final byte[] hbuf;
        private final char[] cbuf;
        private boolean blkmode;
        private int pos;
        private int end;
        private int unread;
        private final ObjectInputStream.PeekInputStream in;
        private final DataInputStream din;
        
        BlockDataInputStream(InputStream inputStream) {
            this.buf = new byte[1024];
            this.hbuf = new byte[5];
            this.cbuf = new char[256];
            this.blkmode = false;
            this.pos = 0;
            this.end = -1;
            this.unread = 0;
            this.in = new PeekInputStream(inputStream);
            this.din = new DataInputStream(this);
        }
        
        boolean setBlockDataMode(boolean blkmode) throws IOException {
            if (this.blkmode == blkmode) {
                return this.blkmode;
            }
            if (blkmode) {
                this.pos = 0;
                this.end = 0;
                this.unread = 0;
            }
            else if (this.pos < this.end) {
                throw new IllegalStateException("unread block data");
            }
            this.blkmode = blkmode;
            return !this.blkmode;
        }
        
        boolean getBlockDataMode() {
            return this.blkmode;
        }
        
        void skipBlockData() throws IOException {
            if (!this.blkmode) {
                throw new IllegalStateException("not in block data mode");
            }
            while (this.end >= 0) {
                this.refill();
            }
            return;
        }
        
        private int readBlockHeader(boolean b) throws IOException {
            if (ObjectInputStream.this.defaultDataEnd) {
                return -1;
            }
            while (true) {
                try {
                    int n = b ? Integer.MAX_VALUE : this.in.available();
                    if (n == 0) {
                        return -2;
                    }
                    int peek = this.in.peek();
                    switch (peek) {
                        case 119: {
                            if (n < 2) {
                                return -2;
                            }
                            this.in.readFully(this.hbuf, 0, 2);
                            return this.hbuf[1] & 0xFF;
                        }
                        case 122: {
                            if (n < 5) {
                                return -2;
                            }
                            this.in.readFully(this.hbuf, 0, 5);
                            int int1 = Bits.getInt(this.hbuf, 1);
                            if (int1 < 0) {
                                throw new StreamCorruptedException("illegal block data header length: " + int1);
                            }
                            return int1;
                        }
                        case 121: {
                            this.in.read();
                            ObjectInputStream.access$1100(ObjectInputStream.this);
                            continue;
                        }
                        default: {
                            if (peek >= 0 && (peek < 112 || peek > 126)) {
                                throw new StreamCorruptedException(String.format("invalid type code: %02X", new Object[] { Integer.valueOf(peek) }));
                            }
                            return -1;
                        }
                    }
                }
                catch (EOFException ex) {
                    throw new StreamCorruptedException("unexpected EOF while reading block data header");
                }
            }
        }
        
        private void refill() throws IOException {
            try {
                do {
                    this.pos = 0;
                    if (this.unread > 0) {
                        int read = this.in.read(this.buf, 0, Math.min(this.unread, 1024));
                        if (read >= 0) {
                            this.end = read;
                            this.unread -= read;
                        }
                        else {
                            throw new StreamCorruptedException("unexpected EOF in middle of data block");
                        }
                    }
                    else {
                        int blockHeader = this.readBlockHeader(true);
                        if (blockHeader >= 0) {
                            this.end = 0;
                            this.unread = blockHeader;
                        }
                        else {
                            this.end = -1;
                            this.unread = 0;
                        }
                    }
                } while (this.pos == this.end);
            }
            catch (IOException ex) {
                this.pos = 0;
                this.end = -1;
                this.unread = 0;
                throw ex;
            }
        }
        
        int currentBlockRemaining() {
            if (this.blkmode) {
                return (this.end >= 0) ? (this.end - this.pos + this.unread) : 0;
            }
            throw new IllegalStateException();
        }
        
        int peek() throws IOException {
            if (this.blkmode) {
                if (this.pos == this.end) {
                    this.refill();
                }
                return (this.end >= 0) ? (this.buf[this.pos] & 0xFF) : -1;
            }
            else {
                return this.in.peek();
            }
        }
        
        byte peekByte() throws IOException {
            int peek = this.peek();
            if (peek < 0) {
                throw new EOFException();
            }
            return (byte)peek;
        }
        
        @Override
        public int read() throws IOException {
            if (this.blkmode) {
                if (this.pos == this.end) {
                    this.refill();
                }
                return (this.end >= 0) ? (this.buf[this.pos++] & 0xFF) : -1;
            }
            else {
                return this.in.read();
            }
        }
        
        @Override
        public int read(byte[] array, int n, int n2) throws IOException {
            return this.read(array, n, n2, false);
        }
        
        @Override
        public long skip(long n) throws IOException {
            while (n > 0L) {
                if (this.blkmode) {
                    if (this.pos == this.end) {
                        this.refill();
                    }
                    if (this.end < 0) {
                        break;
                    }
                    int n2 = (int)Math.min(n, (long)(this.end - this.pos));
                    n -= (long)n2;
                    this.pos += n2;
                }
                else {
                    int read;
                    if ((read = this.in.read(this.buf, 0, (int)Math.min(n, 1024L))) < 0) {
                        break;
                    }
                    n -= (long)read;
                }
            }
            return n - n;
        }
        
        @Override
        public int available() throws IOException {
            if (this.blkmode) {
                if (this.pos == this.end && this.unread == 0) {
                    int blockHeader;
                    while ((blockHeader = this.readBlockHeader(false)) == 0) {}
                    switch (blockHeader) {
                        case -2: {
                            break;
                        }
                        case -1: {
                            this.pos = 0;
                            this.end = -1;
                            break;
                        }
                        default: {
                            this.pos = 0;
                            this.end = 0;
                            this.unread = blockHeader;
                            break;
                        }
                    }
                }
                int n = (this.unread > 0) ? Math.min(this.in.available(), this.unread) : 0;
                return (this.end >= 0) ? (this.end - this.pos + n) : 0;
            }
            else {
                return this.in.available();
            }
        }
        
        @Override
        public void close() throws IOException {
            if (this.blkmode) {
                this.pos = 0;
                this.end = -1;
                this.unread = 0;
            }
            this.in.close();
        }
        
        int read(byte[] array, int n, int n2, boolean b) throws IOException {
            if (n2 == 0) {
                return 0;
            }
            if (this.blkmode) {
                if (this.pos == this.end) {
                    this.refill();
                }
                if (this.end < 0) {
                    return -1;
                }
                int min = Math.min(n2, this.end - this.pos);
                System.arraycopy(this.buf, this.pos, array, n, min);
                this.pos += min;
                return min;
            }
            else if (b) {
                int read = this.in.read(this.buf, 0, Math.min(n2, 1024));
                if (read > 0) {
                    System.arraycopy(this.buf, 0, array, n, read);
                }
                return read;
            }
            else {
                return this.in.read(array, n, n2);
            }
        }
        
        @Override
        public void readFully(byte[] array) throws IOException {
            this.readFully(array, 0, array.length, false);
        }
        
        @Override
        public void readFully(byte[] array, int n, int n2) throws IOException {
            this.readFully(array, n, n2, false);
        }
        
        public void readFully(byte[] array, int n, int i, boolean b) throws IOException {
            while (i > 0) {
                int read = this.read(array, n, i, b);
                if (read < 0) {
                    throw new EOFException();
                }
                n += read;
                i -= read;
            }
        }
        
        @Override
        public int skipBytes(int n) throws IOException {
            return this.din.skipBytes(n);
        }
        
        @Override
        public boolean readBoolean() throws IOException {
            int read = this.read();
            if (read < 0) {
                throw new EOFException();
            }
            return read != 0;
        }
        
        @Override
        public byte readByte() throws IOException {
            int read = this.read();
            if (read < 0) {
                throw new EOFException();
            }
            return (byte)read;
        }
        
        @Override
        public int readUnsignedByte() throws IOException {
            int read = this.read();
            if (read < 0) {
                throw new EOFException();
            }
            return read;
        }
        
        @Override
        public char readChar() throws IOException {
            if (!this.blkmode) {
                this.pos = 0;
                this.in.readFully(this.buf, 0, 2);
            }
            else if (this.end - this.pos < 2) {
                return this.din.readChar();
            }
            int char1 = Bits.getChar(this.buf, this.pos);
            this.pos += 2;
            return char1;
        }
        
        @Override
        public short readShort() throws IOException {
            if (!this.blkmode) {
                this.pos = 0;
                this.in.readFully(this.buf, 0, 2);
            }
            else if (this.end - this.pos < 2) {
                return this.din.readShort();
            }
            int short1 = Bits.getShort(this.buf, this.pos);
            this.pos += 2;
            return short1;
        }
        
        @Override
        public int readUnsignedShort() throws IOException {
            if (!this.blkmode) {
                this.pos = 0;
                this.in.readFully(this.buf, 0, 2);
            }
            else if (this.end - this.pos < 2) {
                return this.din.readUnsignedShort();
            }
            int n = Bits.getShort(this.buf, this.pos) & 0xFFFF;
            this.pos += 2;
            return n;
        }
        
        @Override
        public int readInt() throws IOException {
            if (!this.blkmode) {
                this.pos = 0;
                this.in.readFully(this.buf, 0, 4);
            }
            else if (this.end - this.pos < 4) {
                return this.din.readInt();
            }
            int int1 = Bits.getInt(this.buf, this.pos);
            this.pos += 4;
            return int1;
        }
        
        @Override
        public float readFloat() throws IOException {
            if (!this.blkmode) {
                this.pos = 0;
                this.in.readFully(this.buf, 0, 4);
            }
            else if (this.end - this.pos < 4) {
                return this.din.readFloat();
            }
            float float1 = Bits.getFloat(this.buf, this.pos);
            this.pos += 4;
            return float1;
        }
        
        @Override
        public long readLong() throws IOException {
            if (!this.blkmode) {
                this.pos = 0;
                this.in.readFully(this.buf, 0, 8);
            }
            else if (this.end - this.pos < 8) {
                return this.din.readLong();
            }
            long long1 = Bits.getLong(this.buf, this.pos);
            this.pos += 8;
            return long1;
        }
        
        @Override
        public double readDouble() throws IOException {
            if (!this.blkmode) {
                this.pos = 0;
                this.in.readFully(this.buf, 0, 8);
            }
            else if (this.end - this.pos < 8) {
                return this.din.readDouble();
            }
            double double1 = Bits.getDouble(this.buf, this.pos);
            this.pos += 8;
            return double1;
        }
        
        @Override
        public String readUTF() throws IOException {
            return this.readUTFBody((long)this.readUnsignedShort());
        }
        
        @Override
        public String readLine() throws IOException {
            return this.din.readLine();
        }
        
        void readBooleans(boolean[] array, int i, int n) throws IOException {
            int a = i + n;
            while (i < a) {
                int min;
                if (!this.blkmode) {
                    int min2 = Math.min(a - i, 1024);
                    this.in.readFully(this.buf, 0, min2);
                    min = i + min2;
                    this.pos = 0;
                }
                else {
                    if (this.end - this.pos < 1) {
                        array[i++] = this.din.readBoolean();
                        continue;
                    }
                    min = Math.min(a, i + this.end - this.pos);
                }
                while (i < min) {
                    array[i++] = Bits.getBoolean(this.buf, this.pos++);
                }
            }
        }
        
        void readChars(char[] array, int i, int n) throws IOException {
            int a = i + n;
            while (i < a) {
                int min;
                if (!this.blkmode) {
                    int min2 = Math.min(a - i, 512);
                    this.in.readFully(this.buf, 0, min2 << 1);
                    min = i + min2;
                    this.pos = 0;
                }
                else {
                    if (this.end - this.pos < 2) {
                        array[i++] = this.din.readChar();
                        continue;
                    }
                    min = Math.min(a, i + (this.end - this.pos >> 1));
                }
                while (i < min) {
                    array[i++] = Bits.getChar(this.buf, this.pos);
                    this.pos += 2;
                }
            }
        }
        
        void readShorts(short[] array, int i, int n) throws IOException {
            int a = i + n;
            while (i < a) {
                int min;
                if (!this.blkmode) {
                    int min2 = Math.min(a - i, 512);
                    this.in.readFully(this.buf, 0, min2 << 1);
                    min = i + min2;
                    this.pos = 0;
                }
                else {
                    if (this.end - this.pos < 2) {
                        array[i++] = this.din.readShort();
                        continue;
                    }
                    min = Math.min(a, i + (this.end - this.pos >> 1));
                }
                while (i < min) {
                    array[i++] = Bits.getShort(this.buf, this.pos);
                    this.pos += 2;
                }
            }
        }
        
        void readInts(int[] array, int i, int n) throws IOException {
            int a = i + n;
            while (i < a) {
                int min;
                if (!this.blkmode) {
                    int min2 = Math.min(a - i, 256);
                    this.in.readFully(this.buf, 0, min2 << 2);
                    min = i + min2;
                    this.pos = 0;
                }
                else {
                    if (this.end - this.pos < 4) {
                        array[i++] = this.din.readInt();
                        continue;
                    }
                    min = Math.min(a, i + (this.end - this.pos >> 2));
                }
                while (i < min) {
                    array[i++] = Bits.getInt(this.buf, this.pos);
                    this.pos += 4;
                }
            }
        }
        
        void readFloats(float[] array, int i, int n) throws IOException {
            int n2 = i + n;
            while (i < n2) {
                int n3;
                if (!this.blkmode) {
                    n3 = Math.min(n2 - i, 256);
                    this.in.readFully(this.buf, 0, n3 << 2);
                    this.pos = 0;
                }
                else {
                    if (this.end - this.pos < 4) {
                        array[i++] = this.din.readFloat();
                        continue;
                    }
                    n3 = Math.min(n2 - i, this.end - this.pos >> 2);
                }
                ObjectInputStream.access$1200(this.buf, this.pos, array, i, n3);
                i += n3;
                this.pos += n3 << 2;
            }
        }
        
        void readLongs(long[] array, int i, int n) throws IOException {
            int a = i + n;
            while (i < a) {
                int min;
                if (!this.blkmode) {
                    int min2 = Math.min(a - i, 128);
                    this.in.readFully(this.buf, 0, min2 << 3);
                    min = i + min2;
                    this.pos = 0;
                }
                else {
                    if (this.end - this.pos < 8) {
                        array[i++] = this.din.readLong();
                        continue;
                    }
                    min = Math.min(a, i + (this.end - this.pos >> 3));
                }
                while (i < min) {
                    array[i++] = Bits.getLong(this.buf, this.pos);
                    this.pos += 8;
                }
            }
        }
        
        void readDoubles(double[] array, int i, int n) throws IOException {
            int n2 = i + n;
            while (i < n2) {
                int n3;
                if (!this.blkmode) {
                    n3 = Math.min(n2 - i, 128);
                    this.in.readFully(this.buf, 0, n3 << 3);
                    this.pos = 0;
                }
                else {
                    if (this.end - this.pos < 8) {
                        array[i++] = this.din.readDouble();
                        continue;
                    }
                    n3 = Math.min(n2 - i, this.end - this.pos >> 3);
                }
                ObjectInputStream.access$1300(this.buf, this.pos, array, i, n3);
                i += n3;
                this.pos += n3 << 3;
            }
        }
        
        String readLongUTF() throws IOException {
            return this.readUTFBody(this.readLong());
        }
        
        private String readUTFBody(long b) throws IOException {
            StringBuilder sb;
            if (b > 0L && b < 2147483647L) {
                sb = new StringBuilder(Math.min((int)b, 65535));
            }
            else {
                sb = new StringBuilder();
            }
            if (!this.blkmode) {
                int n = 0;
                this.pos = n;
                this.end = n;
            }
            while (b > 0L) {
                int n2 = this.end - this.pos;
                if (n2 >= 3 || (long)n2 == b) {
                    b -= this.readUTFSpan(sb, b);
                }
                else if (this.blkmode) {
                    b -= (long)this.readUTFChar(sb, b);
                }
                else {
                    if (n2 > 0) {
                        System.arraycopy(this.buf, this.pos, this.buf, 0, n2);
                    }
                    this.pos = 0;
                    this.end = (int)Math.min(1024L, b);
                    this.in.readFully(this.buf, n2, this.end - n2);
                }
            }
            return sb.toString();
        }
        
        private long readUTFSpan(StringBuilder sb, long n) throws IOException {
            int len = 0;
            int pos = this.pos;
            int min = Math.min(this.end - this.pos, 256);
            int n2 = this.pos + ((n > (long)min) ? (min - 2) : ((int)n));
            boolean n3 = false;
            try {
                while (this.pos < n2) {
                    int n4 = this.buf[this.pos++] & 0xFF;
                    switch (n4 >> 4) {
                        case 0:
                        case 1:
                        case 2:
                        case 3:
                        case 4:
                        case 5:
                        case 6:
                        case 7: {
                            this.cbuf[len++] = (char)n4;
                            continue;
                        }
                        case 12:
                        case 13: {
                            int n5 = this.buf[this.pos++];
                            if ((n5 & 0xC0) != 0x80) {
                                throw new UTFDataFormatException();
                            }
                            this.cbuf[len++] = (char)((n4 & 0x1F) << 6 | (n5 & 0x3F) << 0);
                            continue;
                        }
                        case 14: {
                            int n6 = this.buf[this.pos + 1];
                            int n7 = this.buf[this.pos + 0];
                            this.pos += 2;
                            if ((n7 & 0xC0) != 0x80 || (n6 & 0xC0) != 0x80) {
                                throw new UTFDataFormatException();
                            }
                            this.cbuf[len++] = (char)((n4 & 0xF) << 12 | (n7 & 0x3F) << 6 | (n6 & 0x3F) << 0);
                            continue;
                        }
                        default: {
                            throw new UTFDataFormatException();
                        }
                    }
                }
            }
            catch (ArrayIndexOutOfBoundsException ex) {
                n3 = true;
            }
            finally {
                if (n3 || (long)(this.pos - pos) > n) {
                    this.pos = pos + (int)n;
                    throw new UTFDataFormatException();
                }
            }
            sb.append(this.cbuf, 0, len);
            return (long)(this.pos - pos);
        }
        
        private int readUTFChar(StringBuilder sb, long n) throws IOException {
            int n2 = this.readByte() & 0xFF;
            switch (n2 >> 4) {
                case 0:
                case 1:
                case 2:
                case 3:
                case 4:
                case 5:
                case 6:
                case 7: {
                    sb.append((char)n2);
                    return 1;
                }
                case 12:
                case 13: {
                    if (n < 2L) {
                        throw new UTFDataFormatException();
                    }
                    int byte1 = this.readByte();
                    if ((byte1 & 0xC0) != 0x80) {
                        throw new UTFDataFormatException();
                    }
                    sb.append((char)((n2 & 0x1F) << 6 | (byte1 & 0x3F) << 0));
                    return 2;
                }
                case 14: {
                    if (n < 3L) {
                        if (n == 2L) {
                            this.readByte();
                        }
                        throw new UTFDataFormatException();
                    }
                    else {
                        int byte2 = this.readByte();
                        int byte3 = this.readByte();
                        if ((byte2 & 0xC0) != 0x80 || (byte3 & 0xC0) != 0x80) {
                            throw new UTFDataFormatException();
                        }
                        sb.append((char)((n2 & 0xF) << 12 | (byte2 & 0x3F) << 6 | (byte3 & 0x3F) << 0));
                        return 3;
                    }
                }
                default: {
                    throw new UTFDataFormatException();
                }
            }
        }
        
        long getBytesRead() {
            return this.in.getBytesRead();
        }
    }
    
    private static class HandleTable
    {
        private static final byte STATUS_OK = 1;
        private static final byte STATUS_UNKNOWN = 2;
        private static final byte STATUS_EXCEPTION = 3;
        byte[] status;
        Object[] entries;
        ObjectInputStream.HandleTable.HandleList[] deps;
        int lowDep;
        int size;
        
        HandleTable(int n) {
            this.lowDep = -1;
            this.size = 0;
            this.status = new byte[n];
            this.entries = new Object[n];
            this.deps = new ObjectInputStream.HandleTable.HandleList[n];
        }
        
        int assign(Object o) {
            if (this.size >= this.entries.length) {
                this.grow();
            }
            this.status[this.size] = 2;
            this.entries[this.size] = o;
            return this.size++;
        }
        
        void markDependency(int n, int lowDep) {
            if (n == -1 || lowDep == -1) {
                return;
            }
            switch (this.status[n]) {
                case 2: {
                    switch (this.status[lowDep]) {
                        case 1: {
                            break;
                        }
                        case 3: {
                            this.markException(n, (ClassNotFoundException)this.entries[lowDep]);
                            break;
                        }
                        case 2: {
                            if (this.deps[lowDep] == null) {
                                this.deps[lowDep] = new ObjectInputStream.HandleTable.HandleList();
                            }
                            this.deps[lowDep].add(n);
                            if (this.lowDep < 0 || this.lowDep > lowDep) {
                                this.lowDep = lowDep;
                                break;
                            }
                            break;
                        }
                        default: {
                            throw new InternalError();
                        }
                    }
                }
                case 3: {
                    break;
                }
                default: {
                    throw new InternalError();
                }
            }
            return;
        }
        
        void markException(int n, ClassNotFoundException ex) {
            switch (this.status[n]) {
                case 2: {
                    this.status[n] = 3;
                    this.entries[n] = ex;
                    ObjectInputStream.HandleTable.HandleList list = this.deps[n];
                    if (list != null) {
                        for (int size = list.size(), i = 0; i < size; ++i) {
                            this.markException(list.get(i), ex);
                        }
                        this.deps[n] = null;
                        break;
                    }
                }
                case 3: {
                    break;
                }
                default: {
                    throw new InternalError();
                }
            }
        }
        
        void finish(int i) {
            int size;
            if (this.lowDep < 0) {
                size = i + 1;
            }
            else if (this.lowDep >= i) {
                size = this.size;
                this.lowDep = -1;
            }
            else {
                return;
            }
            for (; i < size; ++i) {
                switch (this.status[i]) {
                    case 2: {
                        this.status[i] = 1;
                        this.deps[i] = null;
                        break;
                    }
                    case 1:
                    case 3: {
                        break;
                    }
                    default: {
                        throw new InternalError();
                    }
                }
            }
        }
        
        void setObject(int n, Object o) {
            switch (this.status[n]) {
                case 1:
                case 2: {
                    this.entries[n] = o;
                    break;
                }
                case 3: {
                    break;
                }
                default: {
                    throw new InternalError();
                }
            }
        }
        
        Object lookupObject(int n) {
            return (n != -1 && this.status[n] != 3) ? this.entries[n] : null;
        }
        
        ClassNotFoundException lookupException(int n) {
            return (n != -1 && this.status[n] == 3) ? ((ClassNotFoundException)this.entries[n]) : null;
        }
        
        void clear() {
            Arrays.fill(this.status, 0, this.size, (byte)0);
            Arrays.fill(this.entries, 0, this.size, null);
            Arrays.fill(this.deps, 0, this.size, null);
            this.lowDep = -1;
            this.size = 0;
        }
        
        int size() {
            return this.size;
        }
        
        private void grow() {
            int n = (this.entries.length << 1) + 1;
            byte[] status = new byte[n];
            Object[] entries = new Object[n];
            ObjectInputStream.HandleTable.HandleList[] deps = new ObjectInputStream.HandleTable.HandleList[n];
            System.arraycopy(this.status, 0, status, 0, this.size);
            System.arraycopy(this.entries, 0, entries, 0, this.size);
            System.arraycopy(this.deps, 0, deps, 0, this.size);
            this.status = status;
            this.entries = entries;
            this.deps = deps;
        }
        
        private static class HandleList
        {
            private int[] list;
            private int size;
            
            public HandleList() {
                this.list = new int[4];
                this.size = 0;
            }
            
            public void add(int n) {
                if (this.size >= this.list.length) {
                    int[] list = new int[this.list.length << 1];
                    System.arraycopy(this.list, 0, list, 0, this.list.length);
                    this.list = list;
                }
                this.list[this.size++] = n;
            }
            
            public int get(int n) {
                if (n >= this.size) {
                    throw new ArrayIndexOutOfBoundsException();
                }
                return this.list[n];
            }
            
            public int size() {
                return this.size;
            }
        }
    }
}
